package com.stormtales.notevault.network.sync;

import android.content.Context;
import com.stormtales.notevault.data.repositories.SongRepository;
import com.stormtales.notevault.data.repositories.SongSyncRepository;
import com.stormtales.notevault.network.sync.models.BatchCreateResponse;
import com.stormtales.notevault.network.sync.models.BatchModifyResponse;
import com.stormtales.notevault.ui.gallery.GalleryViewModel;

public class SongSyncModule {
    private SongRepository songRepository;
    private SongSyncRepository songSyncRepository;
    private SongSyncService songSyncService;
    private GalleryViewModel syncViewModel;

    public SongSyncModule(Context context, GalleryViewModel syncViewModel) {
        this.songRepository = new SongRepository(context);
        this.songSyncRepository = new SongSyncRepository(context);
        this.songSyncService = new SongSyncService(context);
        this.syncViewModel = syncViewModel;
    }

    public void syncCreatedSongs() {
        songSyncRepository.loadCreatedSongs(result -> {
            songSyncService.syncCreatedSongs(result, response -> {
                songSyncRepository.markCreatedSongsAsSynced(response);
                syncViewModel.finishCreateSongSyncing();
            });
        });
    }

    public void syncModifiedSongs() {
        songSyncRepository.loadModifiedSongs(result -> {
            songSyncService.syncModifiedSongs(result, response -> {
                songSyncRepository.markModifiedSongsAsSynced(response);
                syncViewModel.finishModifiedSongSyncinc();
            });
        });
    }

    public interface FinishSongCreateSyncingCallback {
        void finishSongSyncing(BatchCreateResponse response);
    }

    public interface FinishSongSyncingCallback {
        void finishSongSyncing(BatchModifyResponse response);
    }
}
