package com.stormtales.notevault.ui.gallery;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.TextView;
import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import com.stormtales.notevault.data.repositories.SongRepository;
import com.stormtales.notevault.databinding.FragmentGalleryBinding;
import com.stormtales.notevault.network.sync.SongSyncModule;
import com.stormtales.notevault.network.sync.SongSyncService;


public class GalleryFragment extends Fragment {

    GalleryViewModel galleryViewModel;
    private FragmentGalleryBinding binding;
    private ProgressBar progress_sync_created_songs;
    private Button sync_created_songs_btn;
    private ProgressBar progress_sync_modified_songs;
    private Button sync_modified_songs_btn;


    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {
        galleryViewModel = new ViewModelProvider(this).get(GalleryViewModel.class);

        binding = FragmentGalleryBinding.inflate(inflater, container, false);
        View root = binding.getRoot();

        progress_sync_created_songs = binding.progressSyncCreatedSongs;
        sync_created_songs_btn = binding.syncCreatedBtn;
        sync_created_songs_btn.setOnClickListener(v -> onSyncCreatedSongs());
        progress_sync_modified_songs = binding.progressSyncModifiedSongs;
        sync_modified_songs_btn = binding.syncModifiedBtn;
        sync_modified_songs_btn.setOnClickListener(v -> onSyncModifiedSogs());

        galleryViewModel.setSongSyncModule(new SongSyncModule(getContext(), galleryViewModel));

        galleryViewModel.getIsCreatedSongSyncing().observe(getViewLifecycleOwner(), isCreatedSyncinc -> {
            if(isCreatedSyncinc) {
                progress_sync_created_songs.setIndeterminate(true);
                sync_created_songs_btn.setEnabled(false);
            } else {
                progress_sync_created_songs.setIndeterminate(false);
                sync_created_songs_btn.setEnabled(true);
            }
        });

        galleryViewModel.getIsModifiedSongSyncing().observe(getViewLifecycleOwner(), isModifiedSyncinc -> {
            if(isModifiedSyncinc) {
                progress_sync_modified_songs.setIndeterminate(true);
                sync_modified_songs_btn.setEnabled(false);
            } else {
                progress_sync_modified_songs.setIndeterminate(false);
                sync_modified_songs_btn.setEnabled(true);
            }
        });
        return root;
    }


    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }

    private void onSyncCreatedSongs() {
        galleryViewModel.startCreateSongSyncing();
    }


    private void onSyncModifiedSogs() {
        galleryViewModel.startModifiedSongSyncinc();
    }
}