package com.stormtales.notevault.ui.home;

import android.util.Log;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.ViewModel;
import com.stormtales.notevault.data.entities.NoteSheet;
import com.stormtales.notevault.data.entities.Song;
import com.stormtales.notevault.data.repositories.SongRepository;
import com.stormtales.notevault.data.sync.SyncStatus;

import java.util.ArrayList;
import java.util.List;

public class HomeViewModel extends ViewModel {

    private MutableLiveData<List<Song>> allSongs;
    private SongRepository songRepository;

    public HomeViewModel() {
        List<Song> currentSongs = new ArrayList<>();
        this.allSongs = new MutableLiveData<>(currentSongs);
        /*this.allSongs.setValue(songRepository.getAllSongs());*/
    }

    public void addSong(Song song, List<NoteSheet> noteSheetList) {
        songRepository.insert(song, internalSongID -> {
            for(NoteSheet noteSheet : noteSheetList) {
                noteSheet.setSongID(Math.toIntExact(internalSongID));
            }
            songRepository.insertNoteSheets(noteSheetList);
        });


        List<Song> currentSongs = allSongs.getValue();
        if(currentSongs == null) {
            currentSongs = new ArrayList<>();
        }

        // Neue Liste erstellen und den Song hinzufügen
        List<Song> updatedSongs = new ArrayList<>(currentSongs);
        updatedSongs.add(song);
        // Neue Liste in MutableLiveData setzen
        allSongs.setValue(updatedSongs);
        Log.d("HomeViewModel", "Song added. Total songs: " + updatedSongs.size());
    }

    public LiveData<List<Song>> getAllSongsLive() {
        return allSongs;
    }

    public void setSongRepository(SongRepository songRepository) {
        this.songRepository = songRepository;
        loadAllSongs();
    }

    public void loadAllSongs() {
        songRepository.getAllSongs(songs->allSongs.setValue(songs));
    }

    public void deleteSong(Song song) {
        songRepository.deleteSong(song);

        List<Song> currentSongs = allSongs.getValue();
        if(currentSongs != null) {
            currentSongs.remove(song);
            allSongs.setValue(currentSongs);
        }
    }

    public void updateSong(Song editedSong) {
        List<Song> currentSongs = allSongs.getValue();
        if(currentSongs != null) {
            for(int i = 0; i < currentSongs.size(); i++) {
                if(currentSongs.get(i).getLocalID() == editedSong.getLocalID()) {
                    currentSongs.set(i, editedSong);
                    break;
                }
            }
            songRepository.updateSong(editedSong);
            allSongs.setValue(currentSongs);
        }
    }

    public SongRepository getSongRepository() {
        return songRepository;
    }
}