package com.stormtales.notevault.ui.home;

import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.TextView;
import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;
import com.stormtales.notevault.R;
import com.stormtales.notevault.data.entities.Song;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

public class SongAdapter extends RecyclerView.Adapter<SongAdapter.SongViewHolder> {

    private List<Song> songList;
    private OnEditSongListener onEditSongListener;
    private OnDeleteSongListener onDeleteSongListener;
    private OnOpenSongListener onOpenSongListener;

    public SongAdapter(List<Song> songList, OnEditSongListener onEditSongListener, OnDeleteSongListener onDeleteSongListener, OnOpenSongListener onOpenSongListener) {
        this.songList = songList;
        this.onEditSongListener = onEditSongListener;
        this.onDeleteSongListener = onDeleteSongListener;
        this.onOpenSongListener = onOpenSongListener;
    }

    public void updateData(List<Song> songs) {
        if (songs == null) {
            this.songList = new ArrayList<>();
        } else {
            this.songList = new ArrayList<>(songs);
        }
        notifyDataSetChanged();
        Log.d("SongAdapter", "Data updated: " + this.songList.size());
    }

    public class SongViewHolder extends RecyclerView.ViewHolder {
        TextView textYear, textComposition, textTitle;
        ImageButton editButton, deleteButton;

        public SongViewHolder(View itemView) {
            super(itemView);
            textYear = itemView.findViewById(R.id.text_year);
            textComposition = itemView.findViewById(R.id.text_composition);
            textTitle = itemView.findViewById(R.id.text_title);
            editButton = itemView.findViewById(R.id.edit_song_button);
            deleteButton = itemView.findViewById(R.id.delete_song_button);

            textTitle.setOnClickListener(v -> {
                int position = getAdapterPosition();
                if (position != RecyclerView.NO_POSITION) {
                    Song song = songList.get(position);
                    onOpenSongListener.onOpenSong(song);
                }
            });
        }
    }

    @NonNull
    @Override
    public @NotNull SongViewHolder onCreateViewHolder(@NonNull @NotNull ViewGroup parent, int i) {
        View itemView = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.item_song, parent, false);
        return new SongViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(@NonNull @NotNull SongAdapter.SongViewHolder holder, int position) {
        Song song = songList.get(position);
        holder.textYear.setText(String.valueOf(song.getYear()));
        holder.textComposition.setText(song.getComposer());
        holder.textTitle.setText(song.getTitle());

        holder.editButton.setOnClickListener(v -> onEditSongListener.onEditSong(song));
        holder.deleteButton.setOnClickListener(v -> onDeleteSongListener.onDeleteSong(song));
    }

    @Override
    public int getItemCount() {
        return songList.size();
    }

    public interface OnEditSongListener {
        void onEditSong(Song song);
    }

    public interface OnDeleteSongListener {
        void onDeleteSong(Song song);
    }

    public interface OnOpenSongListener {
        void onOpenSong(Song song);
    }
}
