package com.stormtales.notevault.ui.songeditor;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.net.Uri;
import android.os.Bundle;
import android.text.Layout;
import android.widget.Button;
import android.widget.EditText;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import androidx.lifecycle.ViewModelProvider;
import com.stormtales.notevault.R;
import com.stormtales.notevault.data.entities.NoteSheet;
import com.stormtales.notevault.data.entities.Song;
import com.stormtales.notevault.data.sync.SyncStatus;
import com.stormtales.notevault.ui.home.HomeViewModel;
import com.stormtales.notevault.utils.NoteSheetsUtil;
import com.stormtales.notevault.utils.Tupel;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;


public class SongEditorDialog extends DialogFragment {

    Dialog dialog;
    private Uri[] noteSheetFiles;
    private HomeViewModel homeViewModel;

    private Song editedSong;

    public SongEditorDialog() {
        // Required empty public constructor
    }


    @NonNull
    @Override
    public @NotNull Dialog onCreateDialog(@Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        LayoutInflater inflater = LayoutInflater.from(getContext());
        View dialogView = inflater.inflate(R.layout.fragment_song_editor_dialog, null);



        dialogView.findViewById(R.id.btnCancel).setOnClickListener(v-> onCancel());
        dialogView.findViewById(R.id.btnSave).setOnClickListener(v -> onSave());

        AlertDialog.Builder builder = new AlertDialog.Builder(getContext());
        builder.setCancelable(false);
        builder.setView(dialogView);

        dialog = builder.create();
        dialog.setCanceledOnTouchOutside(false);


        return dialog;
    }

    @Override
    public void onStart() {
        super.onStart();
        if(editedSong != null) {
            ((EditText) dialog.findViewById(R.id.etTitle)).setText(editedSong.getTitle());
            ((EditText) dialog.findViewById(R.id.etComposer)).setText(editedSong.getComposer());
            ((EditText) dialog.findViewById(R.id.etGenre)).setText(editedSong.getGenre());
            ((EditText) dialog.findViewById(R.id.etYear)).setText(String.valueOf(editedSong.getYear()));
        }
    }

    private void onCancel() {
        dialog.dismiss();
    }

    private void onSave() {
        String title = ((EditText) dialog.findViewById(R.id.etTitle)).getText().toString();
        String composer = ((EditText) dialog.findViewById(R.id.etComposer)).getText().toString();

        String year_string = ((EditText) dialog.findViewById(R.id.etYear)).getText().toString();
        int releaseYear = year_string.isBlank()? 0 : Integer.parseInt(year_string);
        String genre = ((EditText) dialog.findViewById(R.id.etGenre)).getText().toString();

        if(editedSong != null) {
            editedSong.setTitle(title);
            editedSong.setComposer(composer);
            editedSong.setYear(releaseYear);
            editedSong.setGenre(genre);
            editedSong.setSyncStatus(SyncStatus.MODIFIED);
            homeViewModel.updateSong(editedSong);
        } else {
            Song song = new Song(title, composer, genre, releaseYear);

            List<NoteSheet> noteSheetList = new ArrayList<>();
            Context context = this.getContext();
            for(Uri uri : noteSheetFiles) {
                Tupel<String, String> result = NoteSheetsUtil.saveImageInternally(context.getContentResolver(), uri, context.getFilesDir());
                noteSheetList.add(new NoteSheet(result.getValue00(), result.getValue01()));
            }

            homeViewModel.addSong(song, noteSheetList);
        }


        dialog.dismiss();

    }

    public void setNoteSheetFiles(Uri[] noteSheetFiles) {
        this.noteSheetFiles = noteSheetFiles;
    }

    public void setHomeViewModel(HomeViewModel homeViewModel) {
        this.homeViewModel = homeViewModel;
    }

    public void setEditedSong(Song editedSong) {
        this.editedSong = editedSong;
    }
}